@@ -114,8 +114,6 @@ static gboolean      xfce_panel_plugin_remote_event           (XfcePanelPluginPr
 static void          xfce_panel_plugin_set_locked             (XfcePanelPluginProvider          *provider,
                                                                gboolean                          locked);
 static void          xfce_panel_plugin_ask_remove             (XfcePanelPluginProvider          *provider);
-static void          xfce_panel_plugin_take_window_notify     (gpointer                          data,
-                                                               GObject                          *where_the_object_was);
 
 
 
@@ -262,7 +260,8 @@ xfce_panel_plugin_class_init (XfcePanelPluginClass *klass)
    *
    * This signal is emmitted when the Properties entry in the right-click
    * menu is clicked. Plugin writers can use this signal to open a
-   * plugin settings dialog.
+   * plugin settings dialog. It is their responsibility to block/unblock panel
+   * autohide when the dialog is shown/hidden.
    *
    * See also: xfce_panel_plugin_menu_show_configure() and
    *           xfce_titled_dialog_new ().
@@ -1073,6 +1072,8 @@ xfce_panel_plugin_menu_remove (XfcePanelPlugin *plugin)
 
   panel_return_if_fail (XFCE_IS_PANEL_PLUGIN (plugin));
 
+  xfce_panel_plugin_block_autohide (plugin, TRUE);
+
   /* create question dialog (same code is also in panel-preferences-dialog.c) */
   dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
       GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
@@ -1096,6 +1097,8 @@ xfce_panel_plugin_menu_remove (XfcePanelPlugin *plugin)
     }
 
   gtk_widget_destroy (dialog);
+
+  xfce_panel_plugin_block_autohide (plugin, FALSE);
 }
 
 
@@ -1261,8 +1264,8 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
           item = gtk_image_menu_item_new_with_mnemonic (_("_Remove"));
 G_GNUC_END_IGNORE_DEPRECATIONS
-          g_signal_connect_swapped (G_OBJECT (item), "activate",
-              G_CALLBACK (xfce_panel_plugin_menu_remove), plugin);
+          g_signal_connect_object (G_OBJECT (item), "activate",
+              G_CALLBACK (xfce_panel_plugin_menu_remove), plugin, G_CONNECT_SWAPPED);
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
           gtk_widget_show (item);
 
@@ -1717,23 +1720,6 @@ xfce_panel_plugin_ask_remove (XfcePanelPluginProvider *provider)
 
 
 
-static void
-xfce_panel_plugin_take_window_notify (gpointer  data,
-                                      GObject  *where_the_object_was)
-{
-  panel_return_if_fail (GTK_IS_WINDOW (data) || XFCE_IS_PANEL_PLUGIN (data));
-
-  /* release the opposite weak ref */
-  g_object_weak_unref (G_OBJECT (data),
-      xfce_panel_plugin_take_window_notify, where_the_object_was);
-
-  /* destroy the dialog if the plugin was finalized */
-  if (GTK_IS_WINDOW (data))
-    gtk_widget_destroy (GTK_WIDGET (data));
-}
-
-
-
 /**
  * xfce_panel_plugin_get_name:
  * @plugin : an #XfcePanelPlugin.
@@ -2215,12 +2201,8 @@ xfce_panel_plugin_take_window (XfcePanelPlugin *plugin,
   g_return_if_fail (GTK_IS_WINDOW (window));
 
   gtk_window_set_screen (window, gtk_widget_get_screen (GTK_WIDGET (plugin)));
-
-  /* monitor both objects */
-  g_object_weak_ref (G_OBJECT (plugin),
-      xfce_panel_plugin_take_window_notify, window);
-  g_object_weak_ref (G_OBJECT (window),
-      xfce_panel_plugin_take_window_notify, plugin);
+  g_signal_connect_object (plugin, "destroy", G_CALLBACK (gtk_widget_destroy),
+                           window, G_CONNECT_SWAPPED);
 }
 
 
