@@ -260,9 +260,11 @@ panel_plugin_external_finalize (GObject *object)
     {
       /* remove the child watch and don't leave zombies */
       g_source_remove (external->priv->watch_id);
-      g_child_watch_add (external->priv->pid,
-                         (GChildWatchFunc) (void (*)(void)) g_spawn_close_pid,
-                         NULL);
+      external->priv->watch_id = 0;
+      if (external->priv->pid != 0)
+        g_child_watch_add (external->priv->pid,
+                           (GChildWatchFunc) (void (*)(void)) g_spawn_close_pid,
+                           NULL);
     }
 
   panel_plugin_external_queue_free (external);
@@ -481,10 +483,11 @@ panel_plugin_external_child_ask_restart (PanelPluginExternal *external)
         {
           /* remove the child watch and don't leave zombies */
           g_source_remove (external->priv->watch_id);
-          g_child_watch_add (external->priv->pid,
-                             (GChildWatchFunc) (void (*)(void)) g_spawn_close_pid,
-                             NULL);
           external->priv->watch_id = 0;
+          if (external->priv->pid != 0)
+            g_child_watch_add (external->priv->pid,
+                               (GChildWatchFunc) (void (*)(void)) g_spawn_close_pid,
+                               NULL);
         }
 
       /* cleanup the plugin configuration (in PanelApplication) and
@@ -757,8 +760,6 @@ panel_plugin_external_child_watch (GPid     pid,
           xfce_panel_plugin_provider_emit_signal (XFCE_PANEL_PLUGIN_PROVIDER (external),
                                                   PROVIDER_SIGNAL_REMOVE_PLUGIN);
 
-          /* wait until everything is settled before we destroy */
-          panel_utils_destroy_later (GTK_WIDGET (external));
           goto close_pid;
         }
     }
