@@ -26,6 +26,10 @@
 #include <string.h>
 #endif
 
+#include <gdk/gdkx.h>
+
+#include <libxfce4panel/libxfce4panel.h>
+
 #include "sn-button.h"
 #include "sn-icon-box.h"
 #include "sn-util.h"
@@ -43,6 +47,8 @@ static gboolean              sn_button_button_release                (GtkWidget
 static gboolean              sn_button_scroll_event                  (GtkWidget               *widget,
                                                                       GdkEventScroll          *event);
 
+static void                  sn_button_unrealize                     (GtkWidget               *widget);
+
 static void                  sn_button_menu_changed                  (GtkWidget               *widget,
                                                                       SnItem                  *item);
 
@@ -97,6 +103,7 @@ sn_button_class_init (SnButtonClass *klass)
   widget_class->button_press_event = sn_button_button_press;
   widget_class->button_release_event = sn_button_button_release;
   widget_class->scroll_event = sn_button_scroll_event;
+  widget_class->unrealize = sn_button_unrealize;
 }
 
 
@@ -105,6 +112,8 @@ static void
 sn_button_init (SnButton *button)
 {
   GtkCssProvider *css_provider;
+  GdkEventMask    event_mask = GDK_SCROLL_MASK;
+  const gchar    *wm_name;
 
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 
@@ -120,7 +129,12 @@ sn_button_init (SnButton *button)
                                   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
   g_object_unref (css_provider);
 
-  gtk_widget_add_events (GTK_WIDGET (button), GDK_SCROLL_MASK | GDK_SMOOTH_SCROLL_MASK);
+  /* see https://gitlab.xfce.org/xfce/xfwm4/-/issues/641 */
+  wm_name = gdk_x11_screen_get_window_manager_name (gtk_widget_get_screen (GTK_WIDGET (button)));
+  if (g_strcmp0 (wm_name, "Xfwm4") != 0 && g_strcmp0 (wm_name, "unknown") != 0)
+    event_mask |= GDK_SMOOTH_SCROLL_MASK;
+
+  gtk_widget_add_events (GTK_WIDGET (button), event_mask);
 
   button->item = NULL;
   button->config = NULL;
@@ -230,6 +244,7 @@ sn_button_menu_deactivate (GtkWidget *widget,
   }
 
   gtk_widget_unset_state_flags (widget, GTK_STATE_FLAG_ACTIVE);
+  xfce_panel_plugin_block_autohide (XFCE_PANEL_PLUGIN (button->pos_func_data), FALSE);
 }
 
 
@@ -263,6 +278,7 @@ sn_button_button_press (GtkWidget      *widget,
             g_signal_connect_swapped (G_OBJECT (button->menu), "deactivate",
                                       G_CALLBACK (sn_button_menu_deactivate), button);
 
+          xfce_panel_plugin_block_autohide (XFCE_PANEL_PLUGIN (button->pos_func_data), TRUE);
           gtk_menu_popup_at_widget (GTK_MENU (button->menu), widget,
                                     GDK_GRAVITY_NORTH_WEST, GDK_GRAVITY_NORTH_WEST,
                                     (GdkEvent *)event);
@@ -328,10 +344,35 @@ sn_button_scroll_event (GtkWidget      *widget,
   SnButton *button = XFCE_SN_BUTTON (widget);
   gdouble   delta_x, delta_y;
 
-  if (!gdk_event_get_scroll_deltas ((GdkEvent *)event, &delta_x, &delta_y))
+  /* see reference to https://gitlab.xfce.org/xfce/xfwm4/-/issues/641 in init() */
+  if (gtk_widget_get_events (widget) & GDK_SMOOTH_SCROLL_MASK)
     {
-      delta_x = event->delta_x;
-      delta_y = event->delta_y;
+      if (!gdk_event_get_scroll_deltas ((GdkEvent *)event, &delta_x, &delta_y))
+        {
+          delta_x = event->delta_x;
+          delta_y = event->delta_y;
+        }
+    }
+  else
+    {
+      delta_x = delta_y = 0;
+      switch (event->direction)
+        {
+          case GDK_SCROLL_UP:
+            delta_y = -1;
+            break;
+          case GDK_SCROLL_DOWN:
+            delta_y = 1;
+            break;
+          case GDK_SCROLL_RIGHT:
+            delta_x = -1;
+            break;
+          case GDK_SCROLL_LEFT:
+            delta_x = 1;
+            break;
+          default:
+            break;
+        }
     }
 
   if (delta_x != 0 || delta_y != 0)
@@ -348,6 +389,25 @@ sn_button_scroll_event (GtkWidget      *widget,
 
 
 
+static void
+sn_button_unrealize (GtkWidget *widget)
+{
+  SnButton *button = XFCE_SN_BUTTON (widget);
+
+  /*
+   * The button could be hidden without being destroyed, as Blueman does for example when
+   * the bluetooth service is stopped (see issue #391). As the menu is attached to the
+   * button, care must be taken that it does not remain shown while the button is hidden.
+   * This also triggers the "deactivate" signal handler to cleanly end the menu display.
+   */
+  if (button->menu != NULL && gtk_widget_get_visible (button->menu))
+    gtk_menu_shell_deactivate (GTK_MENU_SHELL (button->menu));
+
+  GTK_WIDGET_CLASS (sn_button_parent_class)->unrealize (widget);
+}
+
+
+
 static gboolean
 sn_button_menu_size_changed_idle (gpointer user_data)
 {
